@@ -11,18 +11,19 @@
 #include <QMessageBox>
 #include <QNetworkReply>
 #include <QNetworkRequest>
+#include <QProcessEnvironment>
 #include <QPushButton>
-#include <QSettings>
 #include <QTemporaryFile>
 #include <QUrl>
 
+#include "services/settingsservice.h"
 #include "ui_updatedialog.h"
 
-UpdateDialog::UpdateDialog(QWidget *parent, const QString &changesHtml,
-                           const QString &releaseUrl,
+UpdateDialog::UpdateDialog(QWidget *parent, const QString &changesHtml, const QString &releaseUrl,
                            const QString &releaseVersionString)
     : MasterDialog(parent), ui(new Ui::UpdateDialog) {
     ui->setupUi(this);
+    afterSetupUI();
     ui->downloadProgressBar->hide();
 
 #if !defined(Q_OS_WIN)
@@ -33,8 +34,7 @@ UpdateDialog::UpdateDialog(QWidget *parent, const QString &changesHtml,
         "https://www.qownnotes.org/contributing/get-involved.html"));
 
     // inject some generic CSS styles
-    ui->changeLogEdit->document()->setDefaultStyleSheet(
-        Utils::Misc::genericCSS());
+    ui->changeLogEdit->document()->setDefaultStyleSheet(Utils::Misc::genericCSS());
     //    ui->label_4->setText("<style>" + Utils::Misc::genericCSS() +
     //                                 "</style>" + ui->label_4->text());
 
@@ -51,8 +51,8 @@ UpdateDialog::UpdateDialog(QWidget *parent, const QString &changesHtml,
     _updateButton = new QPushButton(tr("&Update"));
     _updateButton->setProperty("ActionRole", Update);
     _updateButton->setDefault(true);
-    _updateButton->setIcon(QIcon::fromTheme(
-        "svn-update", QIcon(":/icons/breeze-qownnotes/16x16/svn-update.svg")));
+    _updateButton->setIcon(
+        QIcon::fromTheme("svn-update", QIcon(":/icons/breeze-qownnotes/16x16/svn-update.svg")));
     ui->buttonBox->addButton(_updateButton, QDialogButtonBox::ActionRole);
 
 #ifdef Q_OS_LINUX
@@ -66,9 +66,8 @@ UpdateDialog::UpdateDialog(QWidget *parent, const QString &changesHtml,
     }
 #endif
 
-    button->setIcon(QIcon::fromTheme(
-        QStringLiteral("edit-download"),
-        QIcon(":/icons/breeze-qownnotes/16x16/edit-download.svg")));
+    button->setIcon(QIcon::fromTheme(QStringLiteral("edit-download"),
+                                     QIcon(":/icons/breeze-qownnotes/16x16/edit-download.svg")));
     ui->buttonBox->addButton(button, QDialogButtonBox::ActionRole);
 
     button = new QPushButton(tr("&Skip version"));
@@ -83,16 +82,15 @@ UpdateDialog::UpdateDialog(QWidget *parent, const QString &changesHtml,
     button->setToolTip(tr("Don't show this dialog automatically"));
     button->setProperty("ActionRole", Disable);
     button->setDefault(false);
-    button->setIcon(QIcon::fromTheme(QStringLiteral("window-close"),
-                                     QIcon(":/icons/breeze-qownnotes/16x16/"
-                                           "window-close.svg")));
+    button->setIcon(
+        QIcon::fromTheme(QStringLiteral("window-close"), QIcon(":/icons/breeze-qownnotes/16x16/"
+                                                               "window-close.svg")));
     ui->buttonBox->addButton(button, QDialogButtonBox::ActionRole);
 
     button = new QPushButton(tr("&Cancel"), this);
     button->setProperty("ActionRole", Cancel);
-    button->setIcon(QIcon::fromTheme(
-        QStringLiteral("dialog-cancel"),
-        QIcon(":/icons/breeze-qownnotes/16x16/dialog-cancel.svg")));
+    button->setIcon(QIcon::fromTheme(QStringLiteral("dialog-cancel"),
+                                     QIcon(":/icons/breeze-qownnotes/16x16/dialog-cancel.svg")));
     button->setDefault(false);
     ui->buttonBox->addButton(button, QDialogButtonBox::ActionRole);
 
@@ -128,16 +126,14 @@ void UpdateDialog::dialogButtonClicked(QAbstractButton *button) {
 
     switch (actionRole) {
         case Skip: {
-            QSettings settings;
-            settings.setValue(QStringLiteral("skipVersion"),
-                              this->releaseVersionString);
+            SettingsService settings;
+            settings.setValue(QStringLiteral("skipVersion"), this->releaseVersionString);
             qDebug() << "skip version";
             break;
         }
         case Disable: {
-            QSettings settings;
-            settings.setValue(QStringLiteral("disableAutomaticUpdateDialog"),
-                              true);
+            SettingsService settings;
+            settings.setValue(QStringLiteral("disableAutomaticUpdateDialog"), true);
             qDebug() << "disable dialog";
             break;
         }
@@ -145,8 +141,7 @@ void UpdateDialog::dialogButtonClicked(QAbstractButton *button) {
             QString release = qApp->property("release").toString();
             // if the release was build by the CI systems download the new
             // release
-            if ((release == QLatin1String("Travis CI")) ||
-                (release == QLatin1String("AppVeyor"))) {
+            if ((release == QLatin1String("Travis CI")) || (release == QLatin1String("AppVeyor"))) {
                 // download the new release
                 QDesktopServices::openUrl(QUrl(releaseUrl.toUtf8()));
             } else if ((release == QLatin1String("AppImage"))) {
@@ -154,8 +149,8 @@ void UpdateDialog::dialogButtonClicked(QAbstractButton *button) {
                 QDesktopServices::openUrl(QUrl(releaseUrl.toUtf8()));
             } else {
                 // open the installation page
-                QDesktopServices::openUrl(QUrl(
-                    QStringLiteral("https://www.qownnotes.org/installation")));
+                QDesktopServices::openUrl(
+                    QUrl(QStringLiteral("https://www.qownnotes.org/installation")));
             }
             break;
         }
@@ -184,17 +179,14 @@ void UpdateDialog::dialogButtonClicked(QAbstractButton *button) {
 
 #if (QT_VERSION < QT_VERSION_CHECK(5, 9, 0))
             // we really need redirects for GitHub urls!
-            networkRequest.setAttribute(
-                QNetworkRequest::FollowRedirectsAttribute, true);
+            networkRequest.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
 #else
-            networkRequest.setAttribute(
-                QNetworkRequest::RedirectPolicyAttribute, true);
+            networkRequest.setAttribute(QNetworkRequest::RedirectPolicyAttribute, true);
 #endif
 
 #if (QT_VERSION < QT_VERSION_CHECK(5, 15, 0))
             // try to ensure the network is accessible
-            _networkManager->setNetworkAccessible(
-                QNetworkAccessManager::Accessible);
+            _networkManager->setNetworkAccessible(QNetworkAccessManager::Accessible);
 #endif
 
             QNetworkReply *reply = _networkManager->get(networkRequest);
@@ -217,10 +209,11 @@ void UpdateDialog::dialogButtonClicked(QAbstractButton *button) {
 /**
  * Shows the download progress
  */
-void UpdateDialog::releaseDownloadProgress(qint64 bytesReceived,
-                                           qint64 bytesTotal) {
-    ui->downloadProgressBar->setMaximum(static_cast<int>(bytesTotal));
-    ui->downloadProgressBar->setValue(static_cast<int>(bytesReceived));
+void UpdateDialog::releaseDownloadProgress(qint64 bytesReceived, qint64 bytesTotal) {
+    ui->downloadProgressBar->setMaximum(static_cast<int>(bytesTotal / 1000));
+    ui->downloadProgressBar->setValue(static_cast<int>(bytesReceived / 1000));
+    ui->downloadProgressBar->setToolTip(Utils::Misc::toHumanReadableByteSize(bytesReceived) +
+                                        " / " + Utils::Misc::toHumanReadableByteSize(bytesTotal));
 }
 
 /**
@@ -243,9 +236,8 @@ void UpdateDialog::slotReplyFinished(QNetworkReply *reply) {
     qDebug() << __func__ << " - 'data.size': " << data.size();
 
     if (reply->error() != QNetworkReply::NoError) {
-        QMessageBox::critical(
-            0, tr("Download error"),
-            tr("Error while downloading:\n%1").arg(reply->errorString()));
+        QMessageBox::critical(nullptr, tr("Download error"),
+                              tr("Error while downloading:\n%1").arg(reply->errorString()));
 
         qWarning() << tr("network error: %1").arg(reply->errorString());
         _updateButton->setDisabled(false);
@@ -261,17 +253,16 @@ void UpdateDialog::slotReplyFinished(QNetworkReply *reply) {
     QString suffix = "download";
 #endif
 
-    QTemporaryFile *tempFile =
-        new QTemporaryFile(QDir::tempPath() + "/QOwnNotes-XXXXXX." + suffix);
+    auto *tempFile = new QTemporaryFile(QDir::tempPath() + "/QOwnNotes-XXXXXX." + suffix);
 
     // we want to keep the file to be used in the update process
     tempFile->setAutoRemove(false);
 
     // get a temporary file
     if (!tempFile->open()) {
-        QMessageBox::critical(0, tr("File error"),
-                              tr("Could not open temporary file:\n%1")
-                                  .arg(tempFile->errorString()));
+        QMessageBox::critical(
+            nullptr, tr("File error"),
+            tr("Could not open temporary file:\n%1").arg(tempFile->errorString()));
         return;
     }
 
@@ -289,9 +280,8 @@ void UpdateDialog::slotReplyFinished(QNetworkReply *reply) {
     QFile file(filePath);
 
     if (!file.open(QIODevice::WriteOnly)) {
-        QMessageBox::critical(
-            0, tr("File error"),
-            tr("Could not store downloaded file:\n%1").arg(file.errorString()));
+        QMessageBox::critical(nullptr, tr("File error"),
+                              tr("Could not store downloaded file:\n%1").arg(file.errorString()));
         return;
     }
 
@@ -337,14 +327,13 @@ bool UpdateDialog::initializeMacOSUpdateProcess(const QString &releaseUrl) {
     QString applicationsPath = QStringLiteral("/Applications");
 
     if (applicationDirPath.endsWith(appPathPart)) {
-        applicationsPath = Utils::Misc::removeIfEndsWith(
-            std::move(applicationDirPath), appPathPart);
+        applicationsPath =
+            Utils::Misc::removeIfEndsWith(std::move(applicationDirPath), appPathPart);
     }
 
     if (Utils::Gui::question(this, tr("Proceed with update"),
                              tr("Do you want to update and restart QOwnNotes?"),
-                             QStringLiteral("update-and-restart")) !=
-        QMessageBox::Yes) {
+                             QStringLiteral("update-and-restart")) != QMessageBox::Yes) {
         return false;
     }
 
@@ -357,9 +346,8 @@ bool UpdateDialog::initializeMacOSUpdateProcess(const QString &releaseUrl) {
     QFile f(updaterPath);
 
     if (!f.exists()) {
-        QMessageBox::critical(
-            0, tr("Error"),
-            tr("Could not find updater file:\n%1").arg(f.fileName()));
+        QMessageBox::critical(nullptr, tr("Error"),
+                              tr("Could not find updater file:\n%1").arg(f.fileName()));
         return false;
     }
 
@@ -368,25 +356,23 @@ bool UpdateDialog::initializeMacOSUpdateProcess(const QString &releaseUrl) {
     QTextStream ts(&f);
     QString scriptContent = ts.readAll();
     f.close();
-    scriptContent.replace(QLatin1String("\"$QOWNNOTES_RELEASE_URL\""),
-                          "\"" + releaseUrl + "\"");
-    scriptContent.replace(
-        QLatin1String("\"$QOWNNOTES_APPLICATIONS_PATH\""),
-        "\"" + QDir::toNativeSeparators(applicationsPath) + "\"");
+    scriptContent.replace(QLatin1String("\"$QOWNNOTES_RELEASE_URL\""), "\"" + releaseUrl + "\"");
+    scriptContent.replace(QLatin1String("\"$QOWNNOTES_APPLICATIONS_PATH\""),
+                          "\"" + QDir::toNativeSeparators(applicationsPath) + "\"");
 
     // we need a temporary script file with hardcoded parameters because some
     // users were not able to get parameters via the system environment
-    QTemporaryFile *tempFile = new QTemporaryFile(
-        QDir::tempPath() + "/QOwnNotes-Updater-XXXXXX.command");
+    QTemporaryFile *tempFile =
+        new QTemporaryFile(QDir::tempPath() + "/QOwnNotes-Updater-XXXXXX.command");
 
     // we want to keep the file to be used in the update process
     tempFile->setAutoRemove(false);
 
     // get a temporary file
     if (!tempFile->open()) {
-        QMessageBox::critical(0, tr("File error"),
-                              tr("Could not open temporary file:\n%1")
-                                  .arg(tempFile->errorString()));
+        QMessageBox::critical(
+            0, tr("File error"),
+            tr("Could not open temporary file:\n%1").arg(tempFile->errorString()));
         return false;
     }
 
@@ -394,8 +380,7 @@ bool UpdateDialog::initializeMacOSUpdateProcess(const QString &releaseUrl) {
     tempFile->write(scriptContent.toLatin1());
 
     // setting executable permissions to the updater script
-    tempFile->setPermissions(QFile::ExeUser | QFile::ReadUser |
-                             QFile::WriteUser);
+    tempFile->setPermissions(QFile::ExeUser | QFile::ReadUser | QFile::WriteUser);
 
     // file->fileName() only holds a value after file->open()
     QString updaterFilePath = tempFile->fileName();
@@ -430,7 +415,7 @@ bool UpdateDialog::initializeMacOSUpdateProcess(const QString &releaseUrl) {
  * Initializes the Linux update process
  */
 bool UpdateDialog::initializeLinuxUpdateProcess(const QString &filePath) {
-    const QString appPath = qApp->property("arguments").toStringList()[0];
+    const QString appPath = Utils::Misc::applicationPath();
 
     qDebug() << __func__ << " - 'filePath': " << filePath;
     qDebug() << __func__ << " - 'appPath': " << appPath;
@@ -439,10 +424,12 @@ bool UpdateDialog::initializeLinuxUpdateProcess(const QString &filePath) {
     QFileInfo fileInfo(appPath);
 
     if (!fileInfo.isWritable()) {
-        QMessageBox::critical(0, tr("Permission error"),
-          tr("Your QOwnNotes executable '%1' is not writeable! It must be "
-             "writeable by the current user in order to be updated.")
-              .arg(appPath));
+        qCritical() << __func__ << " - 'appPath' is not writable: " << appPath;
+
+        QMessageBox::critical(nullptr, tr("Permission error"),
+                              tr("Your QOwnNotes executable '%1' is not writable! It must be "
+                                 "writable by the current user in order to be updated.")
+                                  .arg(appPath));
 
         return false;
     }
@@ -451,30 +438,36 @@ bool UpdateDialog::initializeLinuxUpdateProcess(const QString &filePath) {
 
     // make the new AppImage executable
     if (!updateFile.setPermissions(updateFile.permissions() | QFileDevice::ExeOwner)) {
-        QMessageBox::critical(0, tr("Permission error"),
-          tr("The temporary file '%1' could not be made executable! "
-            "You need to replace '%2' yourself.")
-              .arg(filePath, appPath));
+        qCritical() << __func__ << " - 'updateFile' could not be made executable: " << filePath;
+
+        QMessageBox::critical(nullptr, tr("Permission error"),
+                              tr("The temporary file '%1' could not be made executable! "
+                                 "You need to replace '%2' yourself.")
+                                  .arg(filePath, appPath));
 
         return false;
     }
 
     // remove the current binary
     if (!file.remove()) {
-        QMessageBox::critical(0, tr("File error"),
-          tr("Your old QOwnNotes executable '%1' could not be removed! "
-                "You need to replace it yourself with '%2'.")
-              .arg(appPath, filePath));
+        qCritical() << __func__ << " - 'file' could not be removed: " << appPath;
+
+        QMessageBox::critical(nullptr, tr("File error"),
+                              tr("Your old QOwnNotes executable '%1' could not be removed! "
+                                 "You need to replace it yourself with '%2'.")
+                                  .arg(appPath, filePath));
 
         return false;
     }
 
     // rename the new AppImage to the path of the current binary
     if (!updateFile.rename(appPath)) {
-        QMessageBox::critical(0, tr("File error"),
-          tr("Your old QOwnNotes executable '%1' could not be replaced "
-                "by the new file '%2'! You need to replace it yourself.")
-              .arg(appPath, filePath));
+        qCritical() << __func__ << " - 'updateFile' could not be renamed: " << filePath;
+
+        QMessageBox::critical(nullptr, tr("File error"),
+                              tr("Your old QOwnNotes executable '%1' could not be replaced "
+                                 "by the new file '%2'! You need to replace it yourself.")
+                                  .arg(appPath, filePath));
 
         return false;
     }
@@ -483,7 +476,10 @@ bool UpdateDialog::initializeLinuxUpdateProcess(const QString &filePath) {
 
     // make the new AppImage executable (2nd attempt)
     if (!appFile.setPermissions(appFile.permissions() | QFileDevice::ExeOwner)) {
-        QMessageBox::critical(0, tr("Permission error"),
+        qCritical() << __func__
+                    << " - 'appFile' could not be made executable after update: " << appPath;
+
+        QMessageBox::critical(nullptr, tr("Permission error"),
                               tr("The app file '%1' could not be made executable! "
                                  "You need to make it executable yourself.")
                                   .arg(appPath));
@@ -513,75 +509,67 @@ bool UpdateDialog::initializeWindowsUpdateProcess(const QString &filePath) {
     QString updaterPath = pathPrefix + "-updater.bat";
     QString unzipPath = pathPrefix + "-unzip.exe";
 
-    QFile updaterSourceFile(QCoreApplication::applicationDirPath() +
-                            "/update.bat");
-    QFile unzipSourceFile(QCoreApplication::applicationDirPath() +
-                          "/unzip.exe");
+    QFile updaterSourceFile(QCoreApplication::applicationDirPath() + "/update.bat");
+    QFile unzipSourceFile(QCoreApplication::applicationDirPath() + "/unzip.exe");
 
     // check if updater script exists
     if (!updaterSourceFile.exists()) {
-        QMessageBox::critical(0, tr("Error"),
-                              tr("Updater script '%1' doesn't exist!")
-                                  .arg(updaterSourceFile.fileName()));
+        QMessageBox::critical(
+            0, tr("Error"),
+            tr("Updater script '%1' doesn't exist!").arg(updaterSourceFile.fileName()));
         return false;
     }
 
     // check if unzip executable exists
     if (!unzipSourceFile.exists()) {
-        QMessageBox::critical(0, tr("Error"),
-                              tr("Updater unzip executable '%1' doesn't exist!")
-                                  .arg(unzipSourceFile.fileName()));
+        QMessageBox::critical(
+            0, tr("Error"),
+            tr("Updater unzip executable '%1' doesn't exist!").arg(unzipSourceFile.fileName()));
         return false;
     }
 
     // copy updater script
     if (!updaterSourceFile.copy(updaterPath)) {
-        QMessageBox::critical(
-            0, tr("Error"),
-            tr("Could not copy updater script '%1' to '%2'!")
-                .arg(updaterSourceFile.fileName(), unzipPath));
+        QMessageBox::critical(nullptr, tr("Error"),
+                              tr("Could not copy updater script '%1' to '%2'!")
+                                  .arg(updaterSourceFile.fileName(), unzipPath));
         return false;
     }
 
     // copy unzip executable script
     if (!unzipSourceFile.copy(unzipPath)) {
-        QMessageBox::critical(
-            0, tr("Error"),
-            tr("Could not copy updater script '%1' to '%2'!")
-                .arg(updaterSourceFile.fileName(), unzipPath));
+        QMessageBox::critical(nullptr, tr("Error"),
+                              tr("Could not copy updater script '%1' to '%2'!")
+                                  .arg(updaterSourceFile.fileName(), unzipPath));
         return false;
     }
 
     // check if updater script exists in temporary folder
     QFile updaterFile(updaterPath);
     if (!updaterFile.exists()) {
-        QMessageBox::critical(
-            0, tr("Error"),
-            tr("Couldn't find updater script: %1").arg(updaterPath));
+        QMessageBox::critical(nullptr, tr("Error"),
+                              tr("Couldn't find updater script: %1").arg(updaterPath));
         return false;
     }
 
     // check if unzip executable exists in temporary folder
     QFile unzipFile(unzipPath);
     if (!unzipFile.exists()) {
-        QMessageBox::critical(
-            0, tr("Error"),
-            tr("Couldn't find unzip executable: %1").arg(unzipPath));
+        QMessageBox::critical(nullptr, tr("Error"),
+                              tr("Couldn't find unzip executable: %1").arg(unzipPath));
         return false;
     }
 
-    if (Utils::Gui::question(
-            this, tr("Proceed with update"),
-            tr("The download is now finished. Do you want to update and "
-               "restart QOwnNotes?"),
-            QStringLiteral("update-and-restart")) != QMessageBox::Yes) {
+    if (Utils::Gui::question(this, tr("Proceed with update"),
+                             tr("The download is now finished. Do you want to update and "
+                                "restart QOwnNotes?"),
+                             QStringLiteral("update-and-restart")) != QMessageBox::Yes) {
         return false;
     }
 
     QStringList parameters(QStringList()
                            << QDir::toNativeSeparators(filePath)
-                           << QDir::toNativeSeparators(
-                                  QCoreApplication::applicationDirPath())
+                           << QDir::toNativeSeparators(QCoreApplication::applicationDirPath())
                            << QDir::toNativeSeparators(unzipPath));
 
     if (Utils::Misc::isInPortableMode()) {
@@ -600,9 +588,7 @@ bool UpdateDialog::initializeWindowsUpdateProcess(const QString &filePath) {
 /**
  * Checks if the update dialog is open
  */
-bool UpdateDialog::isUpdateDialogOpen() {
-    return qApp->property("isUpdateDialogOpen").toBool();
-}
+bool UpdateDialog::isUpdateDialogOpen() { return qApp->property("isUpdateDialogOpen").toBool(); }
 
 /**
  * Sets if the update dialog is open
