@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2022 Patrizio Bekerle -- <patrizio@bekerle.com>
+ * Copyright (c) 2014-2025 Patrizio Bekerle -- <patrizio@bekerle.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,6 +73,9 @@ class FakeVimHandler;
 class WebSocketServerService;
 class QOwnNotesMarkdownTextEdit;
 class CommandBar;
+class WaitingSpinnerWidget;
+class NoteFilePathLabel;
+class NoteRelationScene;
 struct TagHeader;
 
 // forward declaration because of "xxx does not name a type"
@@ -81,8 +84,7 @@ class SettingsDialog;
 
 class MainWindow : public QMainWindow {
     Q_OBJECT
-    Q_PROPERTY(Note currentNote WRITE setCurrentNote MEMBER currentNote NOTIFY
-                   currentNoteChanged)
+    Q_PROPERTY(Note currentNote WRITE setCurrentNote MEMBER currentNote NOTIFY currentNoteChanged)
 
     friend struct FileWatchDisabler;
 
@@ -118,22 +120,21 @@ class MainWindow : public QMainWindow {
 
     void triggerStartupMenuAction();
 
+    void reloadOpenAiControls();
+
     void setCurrentNoteText(QString text);
 
-    void setCurrentNote(Note note, bool updateNoteText = true,
-                        bool updateSelectedNote = true,
+    void setCurrentNote(Note note, bool updateNoteText = true, bool updateSelectedNote = true,
                         bool addPreviousNoteToHistory = true);
 
-    void createNewNote(QString noteName = QString(),
-                       bool withNameAppend = true);
+    void createNewNote(QString noteName = QString(), bool withNameAppend = true);
 
     void doSearchInNote(QString searchText);
 
     const Note &getCurrentNote();
 
-    void createNewNote(
-        QString name, QString text,
-        CreateNewNoteOptions options = CreateNewNoteOption::None);
+    void createNewNote(QString name, QString text,
+                       CreateNewNoteOptions options = CreateNewNoteOption::None);
 
     void openSettingsDialog(int page = 0, bool openScriptRepository = false);
 
@@ -147,7 +148,10 @@ class MainWindow : public QMainWindow {
 
     void enableShowTrashButton();
 
-    void showStatusBarMessage(const QString &message, const int timeout = 4000);
+    void showStatusBarMessage(const QString &message, const QString &symbol = QString(),
+                              int timeout = 4000);
+
+    void showStatusBarMessage(const QString &message, int timeout = 4000);
 
     void handleInsertingFromMimeData(const QMimeData *mimeData);
 
@@ -159,14 +163,16 @@ class MainWindow : public QMainWindow {
 
     QAction *findAction(const QString &objectName);
 
+    void editEncryptedNote();
+
+    void editEncryptedNoteAsync();
+
     void addCustomAction(const QString &identifier, const QString &menuText,
                          const QString &buttonText, const QString &icon,
-                         bool useInNoteEditContextMenu = false,
-                         bool hideButtonInToolbar = false,
+                         bool useInNoteEditContextMenu = false, bool hideButtonInToolbar = false,
                          bool useInNoteListContextMenu = false);
 
-    void addScriptingLabel(const QString &identifier,
-                           const QString &text = QString());
+    void addScriptingLabel(const QString &identifier, const QString &text = QString());
 
     void setScriptingLabelText(const QString &identifier, const QString &text);
 
@@ -176,15 +182,17 @@ class MainWindow : public QMainWindow {
 
     QString selectedNoteTextEditText();
 
-    void linkTagNameToCurrentNote(const QString &tagName,
-                                  bool linkToSelectedNotes = false);
+    void linkTagNameToCurrentNote(const QString &tagName, bool linkToSelectedNotes = false);
+
+    void loadNoteBookmarks();
 
     Q_INVOKABLE void reloadTagTree();
 
     Q_INVOKABLE void reloadNoteSubFolderTree();
 
-    Q_INVOKABLE void buildNotesIndexAndLoadNoteDirectoryList(
-        bool forceBuild = false, bool forceLoad = false, bool reloadTabs = true);
+    Q_INVOKABLE void buildNotesIndexAndLoadNoteDirectoryList(bool forceBuild = false,
+                                                             bool forceLoad = false,
+                                                             bool reloadTabs = true);
 
     QVector<Note> selectedNotes();
 
@@ -220,7 +228,7 @@ class MainWindow : public QMainWindow {
 
     class QOwnNotesMarkdownTextEdit *noteTextEdit();
 
-    void refreshNotePreview();
+    void refreshNotePreview(bool force = false);
 
     Q_INVOKABLE bool removeNoteTab(int index) const;
 
@@ -228,6 +236,8 @@ class MainWindow : public QMainWindow {
 
     Q_INVOKABLE bool jumpToTag(int tagId);
 
+    void enableOpenAiActivitySpinner(bool enable = true);
+
    protected:
     void changeEvent(QEvent *event) override;
 
@@ -244,8 +254,7 @@ class MainWindow : public QMainWindow {
 
     void storeUpdatedNotesToDisk();
 
-    bool changeNoteFolder(const int noteFolderId,
-                          const bool forceChange = false);
+    bool changeNoteFolder(const int noteFolderId, const bool forceChange = false);
 
     void allowNoteEditing();
 
@@ -253,12 +262,14 @@ class MainWindow : public QMainWindow {
 
     void openSelectedNotesInTab();
 
-    void openNoteInTab(const Note& note);
+    void openNoteInTab(const Note &note);
 
     void openCurrentNoteInTab();
 
     Q_INVOKABLE void focusNoteTextEdit();
 
+    void onNavigationWidgetPositionClicked(int position);
+
    private slots:
 
     void on_noteTextEdit_textChanged();
@@ -421,6 +432,8 @@ class MainWindow : public QMainWindow {
 
     void noteViewUpdateTimerSlot();
 
+    void autoReadOnlyModeTimerSlot();
+
     void gitCommitCurrentNoteFolder();
 
     void noteTextSliderValueChanged(int value, bool force = false);
@@ -429,8 +442,7 @@ class MainWindow : public QMainWindow {
 
     void on_tagTreeWidget_itemChanged(QTreeWidgetItem *item, int column);
 
-    void on_tagTreeWidget_currentItemChanged(QTreeWidgetItem *current,
-                                             QTreeWidgetItem *previous);
+    void on_tagTreeWidget_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous);
 
     void on_tagTreeWidget_itemSelectionChanged();
 
@@ -442,7 +454,7 @@ class MainWindow : public QMainWindow {
 
     void on_actionReplace_in_current_note_triggered();
 
-    void onNavigationWidgetPositionClicked(int position);
+    void onBacklinkWidgetNoteClicked(int noteId, QString markdown);
 
     void startNavigationParser();
 
@@ -478,8 +490,7 @@ class MainWindow : public QMainWindow {
 
     void on_actionShow_status_bar_triggered(bool checked);
 
-    void on_noteTreeWidget_currentItemChanged(QTreeWidgetItem *current,
-                                              QTreeWidgetItem *previous);
+    void on_noteTreeWidget_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous);
 
     void on_noteTreeWidget_customContextMenuRequested(const QPoint pos);
 
@@ -529,6 +540,12 @@ class MainWindow : public QMainWindow {
 
     void onWorkspaceComboBoxCurrentIndexChanged(int index);
 
+    void onAiBackendComboBoxCurrentIndexChanged(int index);
+
+    void onAiModelComboBoxCurrentIndexChanged(int index);
+
+    void onAiModelGroupChanged(QAction *action);
+
     void on_actionRemove_current_workspace_triggered();
 
     void on_actionRename_current_workspace_triggered();
@@ -649,7 +666,7 @@ class MainWindow : public QMainWindow {
 
     void onLanguageChanged(QAction *action);
 
-    void onBackendChanged(QAction *action);
+    void onSpellBackendChanged(QAction *action);
 
     void on_actionManage_dictionaries_triggered();
 
@@ -689,14 +706,35 @@ class MainWindow : public QMainWindow {
 
     void on_actionToggle_Always_on_top_triggered();
 
-public:
+    void on_action_Load_Todo_Items_triggered();
+
+    void on_actionInsert_Nextcloud_Deck_card_triggered();
+
+    void on_actionCopy_path_to_note_to_clipboard_triggered();
+
+    void on_actionMove_up_in_subfolder_list_triggered();
+
+    void on_actionMove_down_in_subfolder_list_triggered();
+
+    void on_actionMove_up_in_tag_list_triggered();
+
+    void on_actionMove_down_in_tag_list_triggered();
+
+    void on_actionEnable_AI_toggled(bool arg1);
+
+    void on_navigationTabWidget_currentChanged(int index);
+
+    void on_actionReattach_panels_triggered();
+
+   public:
     /** Settings access **/
     static bool isInDistractionFreeMode();
     void setShowNotesFromAllNoteSubFolders(bool show);
     bool showNotesFromAllNoteSubFolders() const;
+    bool doNoteEditingCheck();
 
     /** Actions **/
-public:
+   public:
     QAction *newNoteAction();
     QAction *reloadNoteFolderAction();
     QAction *insertTextLinkAction();
@@ -704,20 +742,21 @@ public:
     QAction *pasteImageAction();
     QAction *autocompleteAction();
     QAction *splitNoteAtPosAction();
-    QList<QAction*> customTextEditActions();
+    QAction *findNoteAction();
+    QList<QAction *> customTextEditActions();
 
-public:
+   public:
     /** export / print **/
     void printTextDocument(QTextDocument *textDocument);
     void exportNoteAsPDF(QTextDocument *doc);
 
     int getMaxImageWidth() const;
 
-public:
+   public:
     void clearNoteDirectoryWatcher();
     void updateNoteDirectoryWatcher();
 
-private:
+   private:
     Ui::MainWindow *ui;
     QString notesPath;
     QFileSystemWatcher noteDirectoryWatcher;
@@ -734,12 +773,14 @@ private:
     QTimer *noteSaveTimer;
     QTimer *_frequentPeriodicTimer;
     QTimer *_noteViewUpdateTimer;
+    QTimer *_autoReadOnlyModeTimer;
     QTimer *_gitCommitTimer;
     QTimer *_todoListTimer;
     bool _noteViewNeedsUpdate;
     NoteHistory noteHistory;
     QHash<int, NoteHistoryItem> noteBookmarks;
     QPushButton *_updateAvailableButton;
+    NoteFilePathLabel *_noteFilePathLabel;
     QLabel *_noteEditLineNumberLabel;
     QPushButton *_readOnlyButton;
     QPushButton *_leaveDistractionFreeModeButton;
@@ -748,6 +789,7 @@ private:
     QToolBar *_customActionToolbar;
     QToolBar *_insertingToolbar;
     QToolBar *_encryptionToolbar;
+    QToolBar *_aiToolbar;
     QToolBar *_windowToolbar;
     QToolBar *_quitToolbar;
     bool _noteViewIsRegenerated;
@@ -775,6 +817,7 @@ private:
     QDockWidget *_notePreviewDockWidget;
     QDockWidget *_logDockWidget;
     QDockWidget *_scriptingDockWidget;
+    QDockWidget *_noteGraphicsViewDockWidget;
     class LogWidget *_logWidget;
     QWidget *_taggingDockTitleBarWidget;
     QWidget *_noteSubFolderDockTitleBarWidget;
@@ -787,7 +830,11 @@ private:
     QWidget *_notePreviewDockTitleBarWidget;
     QWidget *_logDockTitleBarWidget;
     QWidget *_scriptingDockTitleBarWidget;
+    QWidget *_noteGraphicsViewDockTitleBarWidget;
+    NoteRelationScene *_noteRelationScene;
     QComboBox *_workspaceComboBox;
+    QComboBox *_aiBackendComboBox;
+    QComboBox *_aiModelComboBox;
     QFrame *_noteFolderDockWidgetFrame;
     bool _useNoteFolderButtons;
     bool _noteFolderDockWidgetWasVisible;
@@ -809,6 +856,7 @@ private:
     WebAppClientService *_webAppClientService;
     QActionGroup *_languageGroup;
     QActionGroup *_spellBackendGroup;
+    QActionGroup *_aiModelGroup;
     bool _brokenTagNoteLinksRemoved = false;
 
 #ifdef USE_QLITEHTML
@@ -820,6 +868,9 @@ private:
     bool _scriptUpdateFound = false;
     bool _isMaximizedBeforeFullScreen = false;
     bool _isMinimizedBeforeFullScreen = false;
+    WaitingSpinnerWidget *_openAiActivitySpinner = nullptr;
+
+    void initializeOpenAiActivitySpinner();
 
     void initTreeWidgets();
 
@@ -851,8 +902,7 @@ private:
 
     void loadNoteFolderListMenu();
 
-    void storeRecentNoteFolder(const QString &addFolderName,
-                               const QString &removeFolderName);
+    void storeRecentNoteFolder(const QString &addFolderName, const QString &removeFolderName);
 
     void storeSettings();
 
@@ -880,8 +930,7 @@ private:
 
     void updateNoteEncryptionUI();
 
-    void askForEncryptedNotePasswordIfNeeded(
-        const QString &additionalText = QString());
+    void askForEncryptedNotePasswordIfNeeded(const QString &additionalText = QString());
 
     void showAppMetricsNotificationIfNeeded();
 
@@ -933,15 +982,13 @@ private:
 
     void initToolbars();
 
-    void buildTagTreeForParentItem(QTreeWidgetItem *parent = nullptr,
-                                   bool topLevel = false);
+    void buildTagTreeForParentItem(QTreeWidgetItem *parent = nullptr, bool topLevel = false);
 
     void buildTagMoveMenuTree(QMenu *parentMenu, int parentTagId = 0);
 
     void buildBulkNoteTagMenuTree(QMenu *parentMenu, int parentTagId = 0);
 
-    QTreeWidgetItem *addTagToTagTreeWidget(QTreeWidgetItem *parent,
-                                           const TagHeader &tag);
+    QTreeWidgetItem *addTagToTagTreeWidget(QTreeWidgetItem *parent, const TagHeader &tag);
 
     bool jumpToNoteName(const QString &name);
 
@@ -963,8 +1010,7 @@ private:
 
     void filterNotesByNoteSubFolders();
 
-    bool addNoteToNoteTreeWidget(const Note &note,
-                                 QTreeWidgetItem *parent = nullptr);
+    bool addNoteToNoteTreeWidget(const Note &note, QTreeWidgetItem *parent = nullptr);
 
     QTreeWidgetItem *findNoteInNoteTreeWidget(const Note &note);
 
@@ -977,9 +1023,9 @@ private:
     void buildBulkNoteSubFolderMenuTree(QMenu *parentMenu, bool doCopy = true,
                                         int parentNoteSubFolderId = 0);
 
-    void buildBulkNoteFolderSubFolderMenuTree(
-        QMenu *parentMenu, bool doCopy, const QString &parentNoteSubFolderPath,
-        bool isRoot = true);
+    void buildBulkNoteFolderSubFolderMenuTree(QMenu *parentMenu, bool doCopy,
+                                              const QString &parentNoteSubFolderPath,
+                                              bool isRoot = true);
 
     void moveSelectedNotesToNoteSubFolder(const NoteSubFolder &noteSubFolder);
 
@@ -1039,9 +1085,10 @@ private:
 
     void selectAllNotesInNoteSubFolderTreeWidget() const;
 
-    bool insertAttachment(QFile *file, const QString &title = QString());
+    bool insertAttachment(QFile *file, const QString &title = QString(),
+                          const QString &fileName = QString());
 
-    bool insertTextAsAttachment(const QString &text, const QString &title = QString());
+    bool insertTextAsAttachment(const QString &text);
 
     void updatePanelsSortOrder();
 
@@ -1049,17 +1096,14 @@ private:
 
     void selectAllNotesInTagTreeWidget() const;
 
-    void handleScriptingNoteTagging(Note note, const Tag &tag,
-                                    bool doRemove = false,
+    void handleScriptingNoteTagging(Note note, const Tag &tag, bool doRemove = false,
                                     bool triggerPostMethods = true);
 
     void handleScriptingNotesTagUpdating();
 
-    void handleScriptingNotesTagRenaming(const Tag &tag,
-                                         const QString &newTagName);
+    void handleScriptingNotesTagRenaming(const Tag &tag, const QString &newTagName);
 
-    void handleScriptingNotesTagRemoving(const Tag &tag,
-                                         bool forBulkOperation = false);
+    void handleScriptingNotesTagRemoving(const Tag &tag, bool forBulkOperation = false);
 
     void directoryWatcherWorkaround(bool isNotesDirectoryWasModifiedDisabled,
                                     bool alsoHandleNotesWereModified = false);
@@ -1074,13 +1118,11 @@ private:
 
     int getSelectedNotesCount() const;
 
-    void updateNoteTreeWidgetItem(const Note &note,
-                                  QTreeWidgetItem *noteItem = nullptr);
+    void updateNoteTreeWidgetItem(const Note &note, QTreeWidgetItem *noteItem = nullptr);
 
     void initFakeVim(QOwnNotesMarkdownTextEdit *noteTextEdit);
 
-    void openNotesContextMenu(const QPoint globalPos,
-                              bool multiNoteMenuEntriesOnly = false);
+    void openNotesContextMenu(const QPoint globalPos, bool multiNoteMenuEntriesOnly = false);
 
     void updateCurrentNoteTextHash();
 
@@ -1108,4 +1150,15 @@ private:
     void automaticScriptUpdateCheck();
     void updateJumpToActionsAvailability();
     int getNoteTabIndex(int noteId) const;
+    bool startAutoReadOnlyModeIfEnabled();
+    void updateActionUiEnabled();
+    void buildAiToolbarAndActions();
+    void generateAiBackendComboBox();
+    void generateAiModelComboBox();
+    void generateAiModelMainMenu();
+    void aiModelMainMenuSetCurrentItem();
+    static void handleDockWidgetLocking(QDockWidget *dockWidget);
+    void setupNoteRelationScene();
+    void updateNoteGraphicsView();
+    void addDirectoryToDirectoryWatcher(const QString &path);
 };
